@@ -13,7 +13,7 @@
  *
  * MIT License <https://github.com/nodejs/nan/blob/master/LICENSE.md>
  *
- * Version 2.9.2: current Node 9.5.0, Node 12: 0.12.18, Node 10: 0.10.48, iojs: 3.3.1
+ * Version 2.12.1: current Node 11.4.0, Node 12: 0.12.18, Node 10: 0.10.48, iojs: 3.3.1
  *
  * See https://github.com/nodejs/nan for the latest update to this file
  **********************************************************************************/
@@ -36,6 +36,8 @@
 #define NODE_7_0_MODULE_VERSION  51
 #define NODE_8_0_MODULE_VERSION  57
 #define NODE_9_0_MODULE_VERSION  59
+#define NODE_10_0_MODULE_VERSION 64
+#define NODE_11_0_MODULE_VERSION 67
 
 #ifdef _MSC_VER
 # define NAN_HAS_CPLUSPLUS_11 (_MSC_VER >= 1800)
@@ -91,6 +93,9 @@
 
 namespace Nan {
 
+#define NAN_CONCAT(a, b) NAN_CONCAT_HELPER(a, b)
+#define NAN_CONCAT_HELPER(a, b) a##b
+
 #define NAN_INLINE inline  // TODO(bnoordhuis) Remove in v3.0.0.
 
 #if defined(__GNUC__) && \
@@ -148,6 +153,21 @@ namespace Nan {
 #define NAN_MODULE_INIT(name)                                                  \
     void name(Nan::ADDON_REGISTER_FUNCTION_ARGS_TYPE target)
 
+#if NODE_MAJOR_VERSION >= 10 || \
+    NODE_MAJOR_VERSION == 9 && NODE_MINOR_VERSION >= 3
+#define NAN_MODULE_WORKER_ENABLED(module_name, registration)                   \
+    extern "C" NODE_MODULE_EXPORT void                                         \
+      NAN_CONCAT(node_register_module_v, NODE_MODULE_VERSION)(                 \
+        v8::Local<v8::Object> exports, v8::Local<v8::Value> module,            \
+        v8::Local<v8::Context> context)                                        \
+    {                                                                          \
+        registration(exports);                                                 \
+    }
+#else
+#define NAN_MODULE_WORKER_ENABLED(module_name, registration)                   \
+    NODE_MODULE(module_name, registration)
+#endif
+
 //=== CallbackInfo =============================================================
 
 #include "nan_callbacks.h"  // NOLINT(build/include)
@@ -460,6 +480,116 @@ class TryCatch {
   }
 };
 
+v8::Local<v8::Value> MakeCallback(v8::Local<v8::Object> target,
+                                  v8::Local<v8::Function> func,
+                                  int argc,
+                                  v8::Local<v8::Value>* argv);
+v8::Local<v8::Value> MakeCallback(v8::Local<v8::Object> target,
+                                  v8::Local<v8::String> symbol,
+                                  int argc,
+                                  v8::Local<v8::Value>* argv);
+v8::Local<v8::Value> MakeCallback(v8::Local<v8::Object> target,
+                                  const char* method,
+                                  int argc,
+                                  v8::Local<v8::Value>* argv);
+
+// === AsyncResource ===========================================================
+
+class AsyncResource {
+ public:
+  AsyncResource(
+      v8::Local<v8::String> name
+    , v8::Local<v8::Object> resource = New<v8::Object>()) {
+#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    v8::Isolate* isolate = v8::Isolate::GetCurrent();
+
+    if (resource.IsEmpty()) {
+      resource = New<v8::Object>();
+    }
+
+    context = node::EmitAsyncInit(isolate, resource, name);
+#endif
+  }
+
+  AsyncResource(
+      const char* name
+    , v8::Local<v8::Object> resource = New<v8::Object>()) {
+#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    v8::Isolate* isolate = v8::Isolate::GetCurrent();
+
+    if (resource.IsEmpty()) {
+      resource = New<v8::Object>();
+    }
+
+    v8::Local<v8::String> name_string =
+        New<v8::String>(name).ToLocalChecked();
+    context = node::EmitAsyncInit(isolate, resource, name_string);
+#endif
+  }
+
+  ~AsyncResource() {
+#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    v8::Isolate* isolate = v8::Isolate::GetCurrent();
+    node::EmitAsyncDestroy(isolate, context);
+#endif
+  }
+
+  inline MaybeLocal<v8::Value> runInAsyncScope(
+      v8::Local<v8::Object> target
+    , v8::Local<v8::Function> func
+    , int argc
+    , v8::Local<v8::Value>* argv) {
+#if NODE_MODULE_VERSION < NODE_9_0_MODULE_VERSION
+    return MakeCallback(target, func, argc, argv);
+#else
+    return node::MakeCallback(
+        v8::Isolate::GetCurrent(), target, func, argc, argv, context);
+#endif
+  }
+
+  inline MaybeLocal<v8::Value> runInAsyncScope(
+      v8::Local<v8::Object> target
+    , v8::Local<v8::String> symbol
+    , int argc
+    , v8::Local<v8::Value>* argv) {
+#if NODE_MODULE_VERSION < NODE_9_0_MODULE_VERSION
+    return MakeCallback(target, symbol, argc, argv);
+#else
+    return node::MakeCallback(
+        v8::Isolate::GetCurrent(), target, symbol, argc, argv, context);
+#endif
+  }
+
+  inline MaybeLocal<v8::Value> runInAsyncScope(
+      v8::Local<v8::Object> target
+    , const char* method
+    , int argc
+    , v8::Local<v8::Value>* argv) {
+#if NODE_MODULE_VERSION < NODE_9_0_MODULE_VERSION
+    return MakeCallback(target, method, argc, argv);
+#else
+    return node::MakeCallback(
+        v8::Isolate::GetCurrent(), target, method, argc, argv, context);
+#endif
+  }
+
+ private:
+  NAN_DISALLOW_ASSIGN_COPY_MOVE(AsyncResource)
+#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+  node::async_context context;
+#endif
+};
+
+inline uv_loop_t* GetCurrentEventLoop() {
+#if NODE_MAJOR_VERSION >= 10 || \
+  NODE_MAJOR_VERSION == 9 && NODE_MINOR_VERSION >= 3 || \
+  NODE_MAJOR_VERSION == 8 && NODE_MINOR_VERSION >= 10
+    return node::GetCurrentEventLoop(v8::Isolate::GetCurrent());
+#else
+    return uv_default_loop();
+#endif
+}
+
 //============ =================================================================
 
 /* node 0.12  */
@@ -826,7 +956,7 @@ class TryCatch {
   }
 #endif
 
-  inline v8::Local<v8::Value> MakeCallback(
+  NAN_DEPRECATED inline v8::Local<v8::Value> MakeCallback(
       v8::Local<v8::Object> target
     , v8::Local<v8::Function> func
     , int argc
@@ -836,12 +966,18 @@ class TryCatch {
     return scope.Escape(New(node::MakeCallback(
         v8::Isolate::GetCurrent(), target, func, argc, argv)));
 #else
+# if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    AsyncResource res("nan:makeCallback");
+    return res.runInAsyncScope(target, func, argc, argv)
+        .FromMaybe(v8::Local<v8::Value>());
+# else
     return node::MakeCallback(
         v8::Isolate::GetCurrent(), target, func, argc, argv);
-#endif
+# endif  // NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+#endif  // NODE_MODULE_VERSION < IOJS_3_0_MODULE_VERSION
   }
 
-  inline v8::Local<v8::Value> MakeCallback(
+  NAN_DEPRECATED inline v8::Local<v8::Value> MakeCallback(
       v8::Local<v8::Object> target
     , v8::Local<v8::String> symbol
     , int argc
@@ -851,12 +987,18 @@ class TryCatch {
     return scope.Escape(New(node::MakeCallback(
         v8::Isolate::GetCurrent(), target, symbol, argc, argv)));
 #else
+# if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    AsyncResource res("nan:makeCallback");
+    return res.runInAsyncScope(target, symbol, argc, argv)
+        .FromMaybe(v8::Local<v8::Value>());
+# else
     return node::MakeCallback(
         v8::Isolate::GetCurrent(), target, symbol, argc, argv);
-#endif
+# endif  // NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+#endif  // NODE_MODULE_VERSION < IOJS_3_0_MODULE_VERSION
   }
 
-  inline v8::Local<v8::Value> MakeCallback(
+  NAN_DEPRECATED inline v8::Local<v8::Value> MakeCallback(
       v8::Local<v8::Object> target
     , const char* method
     , int argc
@@ -866,9 +1008,15 @@ class TryCatch {
     return scope.Escape(New(node::MakeCallback(
         v8::Isolate::GetCurrent(), target, method, argc, argv)));
 #else
+# if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    AsyncResource res("nan:makeCallback");
+    return res.runInAsyncScope(target, method, argc, argv)
+        .FromMaybe(v8::Local<v8::Value>());
+# else
     return node::MakeCallback(
         v8::Isolate::GetCurrent(), target, method, argc, argv);
-#endif
+# endif  // NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+#endif  // NODE_MODULE_VERSION < IOJS_3_0_MODULE_VERSION
   }
 
   inline void FatalException(const TryCatch& try_catch) {
@@ -913,7 +1061,13 @@ class Utf8String {
       length_(0), str_(str_st_) {
     HandleScope scope;
     if (!from.IsEmpty()) {
+#if NODE_MAJOR_VERSION >= 10
+      v8::Local<v8::Context> context = GetCurrentContext();
+      v8::Local<v8::String> string =
+          from->ToString(context).FromMaybe(v8::Local<v8::String>());
+#else
       v8::Local<v8::String> string = from->ToString();
+#endif
       if (!string.IsEmpty()) {
         size_t len = 3 * string->Length() + 1;
         assert(len <= INT_MAX);
@@ -923,7 +1077,27 @@ class Utf8String {
         }
         const int flags =
             v8::String::NO_NULL_TERMINATION | imp::kReplaceInvalidUtf8;
+#if NODE_MAJOR_VERSION >= 11
+        length_ = string->WriteUtf8(v8::Isolate::GetCurrent(), str_, static_cast<int>(len), 0, flags);
+#else
+        // See https://github.com/nodejs/nan/issues/832.
+        // Disable the warning as there is no way around it.
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable : 4996)
+#endif
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
         length_ = string->WriteUtf8(str_, static_cast<int>(len), 0, flags);
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
+#endif  // NODE_MAJOR_VERSION < 11
         str_[length_] = '\0';
       }
     }
@@ -1273,93 +1447,6 @@ class Utf8String {
 
 #endif  // NODE_MODULE_VERSION
 
-// === AsyncResource ===========================================================
-
-  class AsyncResource {
-   public:
-    AsyncResource(
-        v8::Local<v8::String> name
-      , v8::Local<v8::Object> resource = New<v8::Object>()) {
-#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
-      v8::Isolate* isolate = v8::Isolate::GetCurrent();
-
-      if (resource.IsEmpty()) {
-        resource = New<v8::Object>();
-      }
-
-      context = node::EmitAsyncInit(isolate, resource, name);
-#endif
-    }
-
-    AsyncResource(
-        const char* name
-      , v8::Local<v8::Object> resource = New<v8::Object>()) {
-#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
-      v8::Isolate* isolate = v8::Isolate::GetCurrent();
-
-      if (resource.IsEmpty()) {
-        resource = New<v8::Object>();
-      }
-
-      v8::Local<v8::String> name_string =
-          New<v8::String>(name).ToLocalChecked();
-      context = node::EmitAsyncInit(isolate, resource, name_string);
-#endif
-    }
-
-    ~AsyncResource() {
-#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
-      v8::Isolate* isolate = v8::Isolate::GetCurrent();
-      node::EmitAsyncDestroy(isolate, context);
-#endif
-    }
-
-    inline MaybeLocal<v8::Value> runInAsyncScope(
-        v8::Local<v8::Object> target
-      , v8::Local<v8::Function> func
-      , int argc
-      , v8::Local<v8::Value>* argv) {
-#if NODE_MODULE_VERSION < NODE_9_0_MODULE_VERSION
-      return MakeCallback(target, func, argc, argv);
-#else
-      return node::MakeCallback(
-          v8::Isolate::GetCurrent(), target, func, argc, argv, context);
-#endif
-    }
-
-    inline MaybeLocal<v8::Value> runInAsyncScope(
-        v8::Local<v8::Object> target
-      , v8::Local<v8::String> symbol
-      , int argc
-      , v8::Local<v8::Value>* argv) {
-#if NODE_MODULE_VERSION < NODE_9_0_MODULE_VERSION
-      return MakeCallback(target, symbol, argc, argv);
-#else
-      return node::MakeCallback(
-          v8::Isolate::GetCurrent(), target, symbol, argc, argv, context);
-#endif
-    }
-
-    inline MaybeLocal<v8::Value> runInAsyncScope(
-        v8::Local<v8::Object> target
-      , const char* method
-      , int argc
-      , v8::Local<v8::Value>* argv) {
-#if NODE_MODULE_VERSION < NODE_9_0_MODULE_VERSION
-      return MakeCallback(target, method, argc, argv);
-#else
-      return node::MakeCallback(
-          v8::Isolate::GetCurrent(), target, method, argc, argv, context);
-#endif
-    }
-
-   private:
-    NAN_DISALLOW_ASSIGN_COPY_MOVE(AsyncResource)
-#if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
-    node::async_context context;
-#endif
-  };
-
 typedef void (*FreeCallback)(char *data, void *hint);
 
 typedef const FunctionCallbackInfo<v8::Value>& NAN_METHOD_ARGS_TYPE;
@@ -1488,10 +1575,16 @@ class Callback {
     , v8::Local<v8::Value> argv[] = 0) const {
 #if NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
     v8::Isolate *isolate = v8::Isolate::GetCurrent();
+# if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    AsyncResource async("nan:Callback:operator()");
+    return Call_(isolate, target, argc, argv, &async)
+        .FromMaybe(v8::Local<v8::Value>());
+# else
     return Call_(isolate, target, argc, argv);
+# endif  // NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
 #else
     return Call_(target, argc, argv);
-#endif
+#endif  //  NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
   }
 
   NAN_DEPRECATED inline v8::Local<v8::Value> operator()(
@@ -1500,12 +1593,19 @@ class Callback {
 #if NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
     v8::Isolate *isolate = v8::Isolate::GetCurrent();
     v8::EscapableHandleScope scope(isolate);
+# if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    AsyncResource async("nan:Callback:operator()");
+    return scope.Escape(Call_(isolate, isolate->GetCurrentContext()->Global(),
+                              argc, argv, &async)
+                            .FromMaybe(v8::Local<v8::Value>()));
+# else
     return scope.Escape(
         Call_(isolate, isolate->GetCurrentContext()->Global(), argc, argv));
+# endif  // NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
 #else
     v8::HandleScope scope;
     return scope.Close(Call_(v8::Context::GetCurrent()->Global(), argc, argv));
-#endif
+#endif  //  NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
   }
 
   inline MaybeLocal<v8::Value> operator()(
@@ -1554,7 +1654,13 @@ class Callback {
      , v8::Local<v8::Value> argv[]) const {
 #if NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
     v8::Isolate *isolate = v8::Isolate::GetCurrent();
+# if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    AsyncResource async("nan:Callback:Call");
+    return Call_(isolate, target, argc, argv, &async)
+        .FromMaybe(v8::Local<v8::Value>());
+# else
     return Call_(isolate, target, argc, argv);
+# endif  // NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
 #else
     return Call_(target, argc, argv);
 #endif
@@ -1569,8 +1675,15 @@ class Callback {
 #if NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
     v8::Isolate *isolate = v8::Isolate::GetCurrent();
     v8::EscapableHandleScope scope(isolate);
+# if NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
+    AsyncResource async("nan:Callback:Call");
+    return scope.Escape(Call_(isolate, isolate->GetCurrentContext()->Global(),
+                              argc, argv, &async)
+                            .FromMaybe(v8::Local<v8::Value>()));
+# else
     return scope.Escape(
         Call_(isolate, isolate->GetCurrentContext()->Global(), argc, argv));
+# endif  // NODE_MODULE_VERSION >= NODE_9_0_MODULE_VERSION
 #else
     v8::HandleScope scope;
     return scope.Close(Call_(v8::Context::GetCurrent()->Global(), argc, argv));
@@ -1626,9 +1739,7 @@ class Callback {
     if (!maybe.ToLocal(&local)) return MaybeLocal<v8::Value>();
     return scope.Escape(local);
   }
-#endif
-
-#if NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
+#elif NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
   v8::Local<v8::Value> Call_(v8::Isolate *isolate
                            , v8::Local<v8::Object> target
                            , int argc
@@ -1679,6 +1790,50 @@ inline MaybeLocal<v8::Value> Call(
   return Call(*callback, recv, argc, argv);
 }
 
+inline MaybeLocal<v8::Value> Call(
+    const Nan::Callback& callback
+  , int argc
+  , v8::Local<v8::Value> argv[]) {
+#if NODE_MODULE_VERSION > NODE_0_10_MODULE_VERSION
+  v8::Isolate* isolate = v8::Isolate::GetCurrent();
+  v8::EscapableHandleScope scope(isolate);
+  return scope.Escape(
+      Call(*callback, isolate->GetCurrentContext()->Global(), argc, argv)
+          .FromMaybe(v8::Local<v8::Value>()));
+#else
+  EscapableHandleScope scope;
+  return scope.Escape(
+      Call(*callback, v8::Context::GetCurrent()->Global(), argc, argv)
+          .FromMaybe(v8::Local<v8::Value>()));
+#endif
+}
+
+inline MaybeLocal<v8::Value> Call(
+    v8::Local<v8::String> symbol
+  , v8::Local<v8::Object> recv
+  , int argc
+  , v8::Local<v8::Value> argv[]) {
+  EscapableHandleScope scope;
+  v8::Local<v8::Value> fn_v =
+      Get(recv, symbol).FromMaybe(v8::Local<v8::Value>());
+  if (fn_v.IsEmpty() || !fn_v->IsFunction()) return v8::Local<v8::Value>();
+  v8::Local<v8::Function> fn = fn_v.As<v8::Function>();
+  return scope.Escape(
+      Call(fn, recv, argc, argv).FromMaybe(v8::Local<v8::Value>()));
+}
+
+inline MaybeLocal<v8::Value> Call(
+    const char* method
+  , v8::Local<v8::Object> recv
+  , int argc
+  , v8::Local<v8::Value> argv[]) {
+  EscapableHandleScope scope;
+  v8::Local<v8::String> method_string =
+      New<v8::String>(method).ToLocalChecked();
+  return scope.Escape(
+      Call(method_string, recv, argc, argv).FromMaybe(v8::Local<v8::Value>()));
+}
+
 /* abstract */ class AsyncWorker {
  public:
   explicit AsyncWorker(Callback *callback_,
@@ -1716,36 +1871,41 @@ inline MaybeLocal<v8::Value> Call(
   inline void SaveToPersistent(
       const char *key, const v8::Local<v8::Value> &value) {
     HandleScope scope;
-    New(persistentHandle)->Set(New(key).ToLocalChecked(), value);
+    Set(New(persistentHandle), New(key).ToLocalChecked(), value).FromJust();
   }
 
   inline void SaveToPersistent(
       const v8::Local<v8::String> &key, const v8::Local<v8::Value> &value) {
     HandleScope scope;
-    New(persistentHandle)->Set(key, value);
+    Set(New(persistentHandle), key, value).FromJust();
   }
 
   inline void SaveToPersistent(
       uint32_t index, const v8::Local<v8::Value> &value) {
     HandleScope scope;
-    New(persistentHandle)->Set(index, value);
+    Set(New(persistentHandle), index, value).FromJust();
   }
 
   inline v8::Local<v8::Value> GetFromPersistent(const char *key) const {
     EscapableHandleScope scope;
     return scope.Escape(
-        New(persistentHandle)->Get(New(key).ToLocalChecked()));
+        Get(New(persistentHandle), New(key).ToLocalChecked())
+        .FromMaybe(v8::Local<v8::Value>()));
   }
 
   inline v8::Local<v8::Value>
   GetFromPersistent(const v8::Local<v8::String> &key) const {
     EscapableHandleScope scope;
-    return scope.Escape(New(persistentHandle)->Get(key));
+    return scope.Escape(
+        Get(New(persistentHandle), key)
+        .FromMaybe(v8::Local<v8::Value>()));
   }
 
   inline v8::Local<v8::Value> GetFromPersistent(uint32_t index) const {
     EscapableHandleScope scope;
-    return scope.Escape(New(persistentHandle)->Get(index));
+    return scope.Escape(
+        Get(New(persistentHandle), index)
+        .FromMaybe(v8::Local<v8::Value>()));
   }
 
   virtual void Execute() = 0;
@@ -1800,7 +1960,7 @@ inline MaybeLocal<v8::Value> Call(
       const char* resource_name = "nan:AsyncBareProgressWorkerBase")
       : AsyncWorker(callback_, resource_name) {
     uv_async_init(
-        uv_default_loop()
+        GetCurrentEventLoop()
       , &async
       , AsyncProgress_
     );
@@ -1841,16 +2001,20 @@ class AsyncBareProgressWorker : public AsyncBareProgressWorkerBase {
       Callback *callback_,
       const char* resource_name = "nan:AsyncBareProgressWorker")
       : AsyncBareProgressWorkerBase(callback_, resource_name) {
+    uv_mutex_init(&async_lock);
   }
 
   virtual ~AsyncBareProgressWorker() {
+    uv_mutex_destroy(&async_lock);
   }
 
   class ExecutionProgress {
     friend class AsyncBareProgressWorker;
    public:
     void Signal() const {
+      uv_mutex_lock(&that_->async_lock);
       uv_async_send(&that_->async);
+      uv_mutex_unlock(&that_->async_lock);
     }
 
     void Send(const T* data, size_t count) const {
@@ -1866,6 +2030,9 @@ class AsyncBareProgressWorker : public AsyncBareProgressWorkerBase {
   virtual void Execute(const ExecutionProgress& progress) = 0;
   virtual void HandleProgressCallback(const T *data, size_t size) = 0;
 
+ protected:
+  uv_mutex_t async_lock;
+
  private:
   void Execute() /*final override*/ {
     ExecutionProgress progress(this);
@@ -1884,21 +2051,19 @@ class AsyncProgressWorkerBase : public AsyncBareProgressWorker<T> {
       const char* resource_name = "nan:AsyncProgressWorkerBase")
       : AsyncBareProgressWorker<T>(callback_, resource_name), asyncdata_(NULL),
         asyncsize_(0) {
-    uv_mutex_init(&async_lock);
   }
 
   virtual ~AsyncProgressWorkerBase() {
-    uv_mutex_destroy(&async_lock);
-
     delete[] asyncdata_;
   }
 
   void WorkProgress() {
-    uv_mutex_lock(&async_lock);
+    uv_mutex_lock(&this->async_lock);
     T *data = asyncdata_;
     size_t size = asyncsize_;
     asyncdata_ = NULL;
-    uv_mutex_unlock(&async_lock);
+    asyncsize_ = 0;
+    uv_mutex_unlock(&this->async_lock);
 
     // Don't send progress events after we've already completed.
     if (this->callback) {
@@ -1915,17 +2080,16 @@ class AsyncProgressWorkerBase : public AsyncBareProgressWorker<T> {
       std::copy(data, data + count, it);
     }
 
-    uv_mutex_lock(&async_lock);
+    uv_mutex_lock(&this->async_lock);
     T *old_data = asyncdata_;
     asyncdata_ = new_data;
     asyncsize_ = count;
-    uv_mutex_unlock(&async_lock);
+    uv_async_send(&this->async);
+    uv_mutex_unlock(&this->async_lock);
 
     delete[] old_data;
-    uv_async_send(&this->async);
   }
 
-  uv_mutex_t async_lock;
   T *asyncdata_;
   size_t asyncsize_;
 };
@@ -2062,7 +2226,7 @@ inline void AsyncExecuteComplete (uv_work_t* req) {
 
 inline void AsyncQueueWorker (AsyncWorker* worker) {
   uv_queue_work(
-      uv_default_loop()
+      GetCurrentEventLoop()
     , &worker->request
     , AsyncExecute
     , reinterpret_cast<uv_after_work_cb>(AsyncExecuteComplete)
@@ -2235,7 +2399,7 @@ SetMethodAux(T recv,
              v8::Local<v8::String> name,
              v8::Local<v8::FunctionTemplate> tpl,
              ...) {
-  recv->Set(name, GetFunction(tpl).ToLocalChecked());
+  Set(recv, name, GetFunction(tpl).ToLocalChecked());
 }
 
 }  // end of namespace imp
@@ -2587,7 +2751,7 @@ struct Tap {
   inline void plan(int i) {
     HandleScope scope;
     v8::Local<v8::Value> arg = New(i);
-    MakeCallback(New(t_), "plan", 1, &arg);
+    Call("plan", New(t_), 1, &arg);
   }
 
   inline void ok(bool isOk, const char *msg = NULL) {
@@ -2595,14 +2759,19 @@ struct Tap {
     v8::Local<v8::Value> args[2];
     args[0] = New(isOk);
     if (msg) args[1] = New(msg).ToLocalChecked();
-    MakeCallback(New(t_), "ok", msg ? 2 : 1, args);
+    Call("ok", New(t_), msg ? 2 : 1, args);
   }
 
   inline void pass(const char * msg = NULL) {
     HandleScope scope;
     v8::Local<v8::Value> hmsg;
     if (msg) hmsg = New(msg).ToLocalChecked();
-    MakeCallback(New(t_), "pass", msg ? 1 : 0, &hmsg);
+    Call("pass", New(t_), msg ? 1 : 0, &hmsg);
+  }
+
+  inline void end() {
+    HandleScope scope;
+    Call("end", New(t_), 0, NULL);
   }
 
  private:
